@@ -6,11 +6,13 @@ import asyncio
 import json
 import logging
 from dataclasses import asdict, dataclass, field
-from datetime import datetime, UTC
+from datetime import UTC, datetime
 from enum import Enum
 from pathlib import Path
 from typing import Any, Dict, List, Literal
 
+from devloop.security.path_validator import PathValidationError, PathValidator
+
 logger = logging.getLogger(__name__)
 
 
@@ -148,16 +150,39 @@ class ContextStore:
     - auto_fixed: Log of silent fixes
     """
 
-    def __init__(self, context_dir: Path | str | None = None):
+    def __init__(
+        self, context_dir: Path | str | None = None, enable_path_validation: bool = True
+    ):
         """
         Initialize context store.
 
         Args:
             context_dir: Directory for context files. Defaults to .devloop/context
+            enable_path_validation: Enable path validation for security (default: True)
         """
         if context_dir is None:
             context_dir = Path.cwd() / ".devloop" / "context"
         self.context_dir = Path(context_dir)
+
+        # Initialize path validator for security
+        project_root = Path.cwd().resolve()
+        self.path_validator = PathValidator(
+            project_root, allow_symlinks=False  # Disallow symlinks for security
+        )
+        self.enable_path_validation = enable_path_validation
+
+        # Validate context directory if validation is enabled
+        if self.enable_path_validation:
+            try:
+                resolved_context_dir = self.context_dir.resolve()
+                if not self.path_validator.is_within_project(resolved_context_dir):
+                    raise PathValidationError(
+                        f"Context directory must be within project: {self.context_dir}"
+                    )
+            except PathValidationError as e:
+                logger.error(f"Invalid context directory: {e}")
+                raise
+
         self._lock = asyncio.Lock()
         self._findings: Dict[Tier, List[Finding]] = {
             Tier.IMMEDIATE: [],
@@ -171,6 +196,8 @@ class ContextStore:
         """Create context directory structure if it doesn't exist."""
         try:
             self.context_dir.mkdir(parents=True, exist_ok=True)
+            # Set proper permissions (rwxr-xr-x) so all agents can write
+            self.context_dir.chmod(0o755)
             logger.info(f"Context directory ready: {self.context_dir}")
         except Exception as e:
             logger.error(f"Failed to create context directory: {e}")
@@ -192,6 +219,19 @@ class ContextStore:
         if isinstance(finding, dict):
             finding = Finding(**finding)
 
+        # Validate file path for security (if enabled)
+        if self.enable_path_validation and finding.file:
+            try:
+                # Check if file path is within project boundaries
+                if not self.path_validator.is_within_project(finding.file):
+                    logger.warning(
+                        f"Ignoring finding with file path outside project: {finding.file}"
+                    )
+                    return
+            except (PathValidationError, OSError) as e:
+                logger.warning(f"Invalid file path in finding: {finding.file}: {e}")
+                return
+
         # Compute relevance score
         if user_context:
             finding.relevance_score = self.compute_relevance(finding, user_context)
@@ -202,6 +242,12 @@ class ContextStore:
         async with self._lock:
             self._findings[tier].append(finding)
             await self._write_tier(tier)
+
+            # Aggressively manage memory: trim old findings when tier gets large
+            # Keep only the most recent findings to prevent unbounded memory growth
+            if len(self._findings[tier]) > 500:  # Per-tier threshold
+                self._trim_tier_memory(tier, keep_count=250)
+
             await self._update_index()
 
         logger.debug(
@@ -255,6 +301,26 @@ class ContextStore:
 
         return min(score, 1.0)
 
+    def _trim_tier_memory(self, tier: Tier, keep_count: int = 250) -> None:
+        """
+        Trim memory by keeping only the most recent findings in a tier.
+
+        Args:
+            tier: Tier to trim
+            keep_count: Number of most recent findings to keep
+        """
+        if len(self._findings[tier]) > keep_count:
+            # Sort by timestamp (most recent first) and keep only the latest
+            sorted_findings = sorted(
+                self._findings[tier], key=lambda f: f.timestamp, reverse=True
+            )
+            self._findings[tier] = sorted_findings[:keep_count]
+            removed = len(sorted_findings) - keep_count
+            logger.debug(
+                f"Trimmed {tier.value} tier: removed {removed} old findings from memory "
+                f"(kept {keep_count} most recent)"
+            )
+
     def assign_tier(self, finding: Finding) -> Tier:
         """
         Assign finding to a tier based on relevance and properties.
@@ -535,7 +601,7 @@ class ContextStore:
         Returns:
             Number of findings removed
         """
-        from datetime import datetime, UTC, timedelta
+        from datetime import UTC, datetime, timedelta
 
         cutoff_time = datetime.now(UTC) - timedelta(hours=hours_to_keep)
         cutoff_iso = cutoff_time.isoformat()
